function [modslope, modlog2spec, phase1spec, phase2spec, phase3spec, scales] = waveletspectra_1D_NDQWT(data, L, wf_q, k1, k2, ismean, isplot)
%  waveletspectra_1D_NDQWT - 1D non-decimated quaternion wavelet spectra for
%  modulus and three phases of quaternion-valued wavelet coefficients
%
%  Usage
%  [modslope, modlener, phase1spec, phase2spec,, phase3spec, scales] = waveletspectra_1D_NDQWT(data, L, wf_q, k1, k2, ismean, isplot)
%
%  input:  data - 1D signal (n by 1 real array)
%          L - degree of coarsest scale
%          wf_q - quaternion wavelet filter
%          k1 -  minimal level for slope calculation (default L)
%          k2 -  maximal level for slope calculation (default J-1)
%          ismean -- 0 for median and 1 for mean, default is mean
%          isplot -- 1 if plot;  0 if not (default 1)
%  output: modslope - scaling slope of modlog2spec by regression fit
%          modlog2spec - log2 of levelwise averages of squared modulus of quaternion-valued wavelet
%                        coefficients
%          phase1spec - levelwise averages of first phases of quaternion-valued wavelet coefficients
%          phase2spec - levelwise averages of second phases of quaternion-valued wavelet coefficients
%          phase3spec - levelwise averages of third phases of quaternion-valued wavelet coefficients
%          scales - scales (L,J-1)
%
%  Copyright (C) -- see WavmatQND/Copyright

J = floor(log2(length(data)));

if nargin == 2,  L=1;   k1=1;  k2=J-1;  ismean=1;  isplot=1; end
if nargin == 3,         k1=L;  k2=J-1;  ismean=1;  isplot=1; end
if nargin == 4,                k2=J-1;  ismean=1;  isplot=1; end
if nargin == 5,                         ismean=1;  isplot=1; end
if nargin == 6,                                    isplot=1; end


[modwddata, phase1wddata, phase2wddata, phase3wddata] = NDQWT_transform1D(data, wf_q); % obtain modulus and three phases after performing NDQWT

scales = L:(J-1);

lener = [];
for i =  L:(J-1)
    help = modwddata( i*length(data)+1 : (i+1)*length(data) ) ;
    if  ismean == 1
        lener = [lener mean(help.^2)];
    elseif ismean == 0
        lener = [lener median(help.^2)];
    else error('not known average of energies. Use mean (ismean=1) or median (ismean=0)')
    end
end

modlog2spec = log2(lener);
yy = modlog2spec(round(k1-L+1):round(k2-L+1));

aa =polyfit([k1:k2], yy, 1);
modslope = aa(1);
cc = polyval(aa,[k1:k2]);

if isplot==1
    %--- set plotting parameters -------
    figure(1);
    lw = 2;
    set(0, 'DefaultAxesFontSize', 15);
    fs = 15;
    msize = 6;
    plot(scales, modlog2spec,  'linewidth', lw)
    hold on
    plot(scales, modlog2spec, 'o', 'markersize', msize)
    plot(k1:k2, yy, 'r-', 'linewidth', lw)
    plot(k1:k2, cc + 1,'g:','linewidth', lw)
    text( k1,cc(1)+0.5, num2cell(modslope) )
    xlabel('dyadic level','fontweight','bold','fontsize',fs)
    ylabel('log spectrum','fontweight','bold','fontsize',fs)
    axis tight
    hold off
end



lener2 = [];
for i =  L:(J-1)
    help = phase1wddata( i*length(data)+1 : (i+1)*length(data) ) ;
    if  ismean == 1
        lener2 = [lener2 mean(help)];
    elseif ismean == 0
        lener2 = [lener2 median(help)];
    else error('not known average of energies. Use mean (ismean=1) or median (ismean=0)')
    end
end

phase1spec = lener2;

if isplot==1
    %--- set plotting parameters -------
    figure(2);
    lw = 2;
    set(0, 'DefaultAxesFontSize', 15);
    fs = 15;
    msize = 6;
    hold on
    plot(scales, phase1spec, 'o', 'markersize', msize)
    xlabel('dyadic level','fontweight','bold','fontsize',fs)
    ylabel('avg of first phases','fontweight','bold','fontsize',fs)
    axis([0 J-1 -1 1])
    hold off
end



lener3 = [];
for i =  L:(J-1)
    help = phase2wddata( i*length(data)+1 : (i+1)*length(data) ) ;
    if  ismean == 1
        lener3 = [lener3 mean(help)];
    elseif ismean == 0
        lener3 = [lener3 median(help)];
    else error('not known average of energies. Use mean (ismean=1) or median (ismean=0)')
    end
end

phase2spec = lener3;

if isplot==1
    %--- set plotting parameters -------
    figure(3);
    lw = 2;
    set(0, 'DefaultAxesFontSize', 15);
    fs = 15;
    msize = 6;
    hold on
    plot(scales, phase2spec, 'o', 'markersize', msize)
    xlabel('dyadic level','fontweight','bold','fontsize',fs)
    ylabel('avg of second phases','fontweight','bold','fontsize',fs)
    axis([0 J-1 -1 1])
    hold off
end



lener4 = [];
for i =  L:(J-1)
    help = phase3wddata( i*length(data)+1 : (i+1)*length(data) ) ;
    if  ismean == 1
        lener4 = [lener4 mean(help)];
    elseif ismean == 0
        lener4 = [lener4 median(help)];
    else error('not known average of energies. Use mean (ismean=1) or median (ismean=0)')
    end
end

phase3spec = lener4;

if isplot==1
    %--- set plotting parameters -------
    figure(4);
    lw = 2;
    set(0, 'DefaultAxesFontSize', 15);
    fs = 15;
    msize = 6;
    hold on
    plot(scales, phase3spec, 'o', 'markersize', msize)
    xlabel('dyadic level','fontweight','bold','fontsize',fs)
    ylabel('avg of third phases','fontweight','bold','fontsize',fs)
    axis([0 J-1 -1 1])
    hold off
end
